@@ -1,8 +1,11 @@
-`msboot` <- function(theta,data,trans,B=5,id="id",verbose=0,...)
+`msboot` <- function(theta,data,B=5,id="id",verbose=0,...)
 {
+    if (!inherits(data, "msdata"))
+        stop("'data' must be a 'msdata' object")
+    trans <- attr(data, "trans")
     ids <- unique(data[[id]])
     n <- length(ids)
-    th <- theta(data,trans,...) # actually only used to get the length
+    th <- theta(data,...) # actually only used to get the length
     res <- matrix(NA,length(th),B)
     for (b in 1:B) {
         if (verbose>0) {
@@ -18,7 +21,7 @@
             print(events(bootdata))
             cat("applying theta ...")
         }
-        thstar <- theta(bootdata,trans,...)
+        thstar <- theta(bootdata,...)
         res[,b] <- thstar
     }
     if (verbose) cat("\n")
